@@ -1,11 +1,11 @@
 #' @include provider.R
 #' @include types.R
 
-method(as_json, list(Provider, TypeBasic)) <- function(provider, x) {
+method(as_json, list(Provider, TypeBasic)) <- function(provider, x, ...) {
   list(type = x@type, description = x@description %||% "")
 }
 
-method(as_json, list(Provider, TypeEnum)) <- function(provider, x) {
+method(as_json, list(Provider, TypeEnum)) <- function(provider, x, ...) {
   list(
     type = "string",
     description = x@description %||% "",
@@ -13,11 +13,11 @@ method(as_json, list(Provider, TypeEnum)) <- function(provider, x) {
   )
 }
 
-method(as_json, list(Provider, TypeObject)) <- function(provider, x) {
+method(as_json, list(Provider, TypeObject)) <- function(provider, x, ...) {
   names <- names2(x@properties)
   required <- map_lgl(x@properties, function(prop) prop@required)
 
-  properties <- as_json(provider, x@properties)
+  properties <- as_json(provider, x@properties, ...)
   names(properties) <- names
 
   list(
@@ -29,14 +29,14 @@ method(as_json, list(Provider, TypeObject)) <- function(provider, x) {
   )
 }
 
-method(as_json, list(Provider, TypeArray)) <- function(provider, x) {
+method(as_json, list(Provider, TypeArray)) <- function(provider, x, ...) {
   list(
     type = "array",
     description = x@description %||% "",
-    items = as_json(provider, x@items)
+    items = as_json(provider, x@items, ...)
   )
 }
 
-method(as_json, list(Provider, TypeJsonSchema)) <- function(provider, x) {
+method(as_json, list(Provider, TypeJsonSchema)) <- function(provider, x, ...) {
   x@json
 }
