@@ -247,7 +247,15 @@ tool_results_as_turn <- function(results) {
   if (!any(is_tool_result)) {
     return(NULL)
   }
-  Turn("user", contents = results[is_tool_result])
+  UserTurn(contents = results[is_tool_result])
+}
+
+turn_split_tool_results <- function(turn) {
+  is_result <- map_lgl(turn@contents, is_tool_result)
+  list(
+    tool_results = turn@contents[is_result],
+    contents = turn@contents[!is_result]
+  )
 }
 
 turn_get_tool_errors <- function(turn = NULL) {
@@ -328,7 +336,7 @@ maybe_echo_tool <- function(x, echo = "output") {
   } else {
     icon <- cli::col_green(cli::symbol$record)
     header <- ""
-    value <- tool_string(x)
+    value <- tool_string_preview(x)
   }
 
   value <- cli::style_italic(value)
