@@ -22,17 +22,13 @@ chat_perform <- function(
 
   switch(
     mode,
-    "value" = chat_perform_value(provider, req),
+    "value" = req_perform(req),
     "stream" = chat_perform_stream(provider, req),
-    "async-value" = chat_perform_async_value(provider, req),
+    "async-value" = req_perform_promise(req),
     "async-stream" = chat_perform_async_stream(provider, req)
   )
 }
 
-chat_perform_value <- function(provider, req) {
-  resp_body_json(req_perform(req))
-}
-
 on_load(
   chat_perform_stream <- coro::generator(function(provider, req) {
     resp <- req_perform_connection(req)
@@ -50,10 +46,6 @@ on_load(
   })
 )
 
-chat_perform_async_value <- function(provider, req) {
-  promises::then(req_perform_promise(req), resp_body_json)
-}
-
 on_load(
   chat_perform_async_stream <- coro::async_generator(function(provider, req) {
     resp <- req_perform_connection(req, blocking = FALSE)
@@ -101,12 +93,6 @@ ellmer_req_robustify <- function(req, is_transient = NULL, after = NULL) {
   req
 }
 
-ellmer_req_credentials <- function(req, credentials_fun) {
-  # TODO: simplify once req_headers_redacted() supports !!!
-  credentials <- credentials_fun()
-  req_headers(req, !!!credentials, .redact = names(credentials))
-}
-
 ellmer_req_user_agent <- function(req, override = "") {
   ua <- if (identical(override, "")) ellmer_user_agent() else override
   req_user_agent(req, ua)
