@@ -7,81 +7,136 @@ NULL
 #' Chat with an Anthropic Claude model
 #'
 #' @description
-#' [Anthropic](https://www.anthropic.com) provides a number of chat based
-#' models under the [Claude](https://www.anthropic.com/claude) moniker.
-#' Note that a Claude Pro membership does not give you the ability to call
-#' models via the API; instead, you will need to sign up (and pay for) a
-#' [developer account](https://console.anthropic.com/).
+#' [Anthropic](https://www.anthropic.com) provides a number of chat based models
+#' under the [Claude](https://claude.com/product/overview) moniker. Note that a
+#' Claude Pro membership does not give you the ability to call models via the
+#' API; instead, you will need to sign up (and pay for) a
+#' [developer account](https://platform.claude.com/).
+#'
+#' # Caching
+#'
+#' Caching with Claude is a bit more complicated than other providers but we
+#' believe that on average it will save you both money and time, so we have
+#' enabled it by default. With other providers, like OpenAI and Google,
+#' you only pay for cache reads, which cost 10% of the normal price. With
+#' Claude, you also pay for cache writes, which cost 125% of the normal price
+#' for 5 minute caching and 200% of the normal price for 1 hour caching.
+#'
+#' How does this affect the total cost of a conversation? Imagine the first
+#' turn sends 1000 input tokens and receives 200 output tokens. The second
+#' turn must first send both the input and output from the previous turn
+#' (1200 tokens). It then sends a further 1000 tokens and receives 200 tokens
+#' back.
+#'
+#' To compare the prices of these two approaches we can ignore the cost of
+#' output tokens, because they are the same for both. How much will the input
+#' tokens cost? If we don't use caching, we send 1000 tokens in the first turn
+#' and 2200 (1000 + 200 + 1000) tokens in the second turn for a total of 3200
+#' tokens. If we use caching, we'll send (the equivalent of) 1000 * 1.25 = 1250
+#' tokens in the first turn. In the second turn, 1000 of the input tokens will
+#' be cached so the total cost is 1000 * 0.1 + (200 + 1000) * 1.25 = 1600
+#' tokens. That makes a total of 2850 tokens, i.e. 11% fewer tokens,
+#' decreasing the overall cost.
+#'
+#' Obviously, the details will vary from conversation to conversation, but
+#' if you have a large system prompt that you re-use many times you should
+#' expect to see larger savings. You can see exactly how many input and
+#' cache input tokens each turn uses, along with the total cost,
+#' with `chat$get_tokens()`. If you don't see savings for your use case, you can
+#' suppress caching with `cache = "none"`.
+#'
+#' I know this is already quite complicated, but there's one final wrinkle:
+#' Claude will only cache longer prompts, with caching requiring at least
+#' 1024-4096 tokens, depending on the model. So don't be surprised it if you
+#' don't see any differences with caching if you have a short prompt.
+#'
+#' See all the details at
+#' <https://docs.claude.com/en/docs/build-with-claude/prompt-caching>.
 #'
 #' @inheritParams chat_openai
 #' @inherit chat_openai return
-#' @param model `r param_model("claude-sonnet-4-20250514", "anthropic")`
-#' @param api_key `r api_key_param("ANTHROPIC_API_KEY")`
-#' @param max_tokens Maximum number of tokens to generate before stopping.
+#' @param model `r param_model("claude-sonnet-4-5-20250929", "anthropic")`
+#' @param api_key `r lifecycle::badge("deprecated")` Use `credentials` instead.
+#' @param credentials `r api_key_param("ANTHROPIC_API_KEY")`
+#' @param base_url The base URL to the endpoint; the default is Claude's
+#'   public API.
+#' @param cache How long to cache inputs? Defaults to "5m" (five minutes).
+#'   Set to "none" to disable caching or "1h" to cache for one hour.
+#'
+#'   See details below.
 #' @param beta_headers Optionally, a character vector of beta headers to opt-in
 #'   claude features that are still in beta.
+#' @param api_headers Named character vector of arbitrary extra headers appended
+#'   to every chat API call.
 #' @family chatbots
 #' @export
-#' @examplesIf has_credentials("claude")
+#' @examples
+#' \dontshow{ellmer:::vcr_example_start("chat_anthropic")}
 #' chat <- chat_anthropic()
 #' chat$chat("Tell me three jokes about statisticians")
+#' \dontshow{ellmer:::vcr_example_end()}
 chat_anthropic <- function(
   system_prompt = NULL,
   params = NULL,
-  max_tokens = deprecated(),
   model = NULL,
+  cache = c("5m", "1h", "none"),
   api_args = list(),
   base_url = "https://api.anthropic.com/v1",
   beta_headers = character(),
-  api_key = anthropic_key(),
+  api_key = NULL,
+  credentials = NULL,
+  api_headers = character(),
   echo = NULL
 ) {
   echo <- check_echo(echo)
 
-  model <- set_default(model, "claude-sonnet-4-20250514")
+  model <- set_default(model, "claude-sonnet-4-5-20250929")
+  cache <- arg_match(cache)
 
-  params <- params %||% params()
-  if (lifecycle::is_present(max_tokens)) {
-    lifecycle::deprecate_warn(
-      when = "0.2.0",
-      what = "chat_anthropic(max_tokens)",
-      with = "chat_anthropic(params)"
-    )
-    params$max_tokens <- max_tokens
-  }
+  credentials <- as_credentials(
+    "chat_anthropic",
+    function() anthropic_key(),
+    credentials = credentials,
+    api_key = api_key
+  )
 
   provider <- ProviderAnthropic(
     name = "Anthropic",
     model = model,
     params = params %||% params(),
     extra_args = api_args,
+    extra_headers = api_headers,
     base_url = base_url,
     beta_headers = beta_headers,
-    api_key = api_key
+    credentials = credentials,
+    cache = cache
   )
 
   Chat$new(provider = provider, system_prompt = system_prompt, echo = echo)
 }
 
+#' @rdname chat_anthropic
+#' @export
+chat_claude <- chat_anthropic
+
 chat_anthropic_test <- function(
   ...,
-  model = "claude-3-5-sonnet-latest",
-  params = NULL
+  model = "claude-sonnet-4-5-20250929",
+  params = NULL,
+  echo = "none"
 ) {
   params <- params %||% params()
-  if (is_testing()) {
-    params$temperature <- params$temperature %||% 0
-  }
+  params$temperature <- params$temperature %||% 0
 
-  chat_anthropic(model = model, params = params, ...)
+  chat_anthropic(model = model, params = params, ..., echo = echo)
 }
 
 ProviderAnthropic <- new_class(
   "ProviderAnthropic",
   parent = Provider,
   properties = list(
-    prop_redacted("api_key"),
-    beta_headers = class_character
+    beta_headers = class_character,
+    cache = prop_string()
   )
 )
 
@@ -97,15 +152,13 @@ method(base_request, ProviderAnthropic) <- function(provider) {
   # <https://docs.anthropic.com/en/api/versioning>
   req <- req_headers(req, `anthropic-version` = "2023-06-01")
   # <https://docs.anthropic.com/en/api/getting-started#authentication>
-  req <- req_headers_redacted(req, `x-api-key` = provider@api_key)
+  req <- ellmer_req_credentials(req, provider@credentials(), "x-api-key")
+
   # <https://docs.anthropic.com/en/api/rate-limits>
-  req <- req_retry(
-    req,
-    # <https://docs.anthropic.com/en/api/errors#http-errors>
-    is_transient = function(resp) resp_status(resp) %in% c(429, 503, 529),
-    max_tries = 2
-  )
-  req <- ellmer_req_timeout(req, stream)
+  # <https://docs.anthropic.com/en/api/errors#http-errors>
+  req <- ellmer_req_robustify(req, is_transient = function(resp) {
+    resp_status(resp) %in% c(429, 503, 529)
+  })
 
   if (length(provider@beta_headers) > 0) {
     req <- req_headers(req, `anthropic-beta` = provider@beta_headers)
@@ -134,18 +187,22 @@ method(chat_body, ProviderAnthropic) <- function(
   tools = list(),
   type = NULL
 ) {
-  if (length(turns) >= 1 && is_system_prompt(turns[[1]])) {
-    system <- turns[[1]]@text
+  if (length(turns) >= 1 && is_system_turn(turns[[1]])) {
+    system <- list(list(type = "text", text = turns[[1]]@text))
+    # Always cache system prompt
+    system[[1]]$cache_control <- cache_control(provider)
   } else {
     system <- NULL
   }
 
-  messages <- compact(as_json(provider, turns))
+  is_last <- seq_along(turns) == length(turns)
+  messages <- compact(map2(turns, is_last, function(turn, is_last) {
+    as_json(provider, turn, is_last = is_last)
+  }))
 
   if (!is.null(type)) {
     tool_def <- ToolDef(
-      fun = function(...) {
-      },
+      function(...) {},
       name = "_structured_tool_call",
       description = "Extract structured data",
       arguments = type_object(data = type)
@@ -159,6 +216,15 @@ method(chat_body, ProviderAnthropic) <- function(
   tools <- as_json(provider, unname(tools))
 
   params <- chat_params(provider, provider@params)
+  if (has_name(params, "budget_tokens")) {
+    thinking <- list(
+      type = "enabled",
+      budget_tokens = params$budget_tokens
+    )
+    params$budget_tokens <- NULL
+  } else {
+    thinking <- NULL
+  }
 
   compact(list2(
     model = provider@model,
@@ -167,6 +233,7 @@ method(chat_body, ProviderAnthropic) <- function(
     stream = stream,
     tools = tools,
     tool_choice = tool_choice,
+    thinking = thinking,
     !!!params
   ))
 }
@@ -179,7 +246,8 @@ method(chat_params, ProviderAnthropic) <- function(provider, params) {
       top_p = "top_p",
       top_k = "top_k",
       max_tokens = "max_tokens",
-      stop_sequences = "stop_sequences"
+      stop_sequences = "stop_sequences",
+      budget_tokens = "reasoning_tokens"
     )
   )
 
@@ -207,7 +275,7 @@ method(stream_parse, ProviderAnthropic) <- function(provider, event) {
 }
 method(stream_text, ProviderAnthropic) <- function(provider, event) {
   if (event$type == "content_block_delta") {
-    event$delta$text
+    event$delta$text %||% event$delta$thinking
   }
 }
 method(stream_merge_chunks, ProviderAnthropic) <- function(
@@ -245,6 +313,12 @@ method(stream_merge_chunks, ProviderAnthropic) <- function(
     result$stop_reason <- chunk$delta$stop_reason
     result$stop_sequence <- chunk$delta$stop_sequence
     result$usage$output_tokens <- chunk$usage$output_tokens
+  } else if (chunk$delta$type == "citations_delta") {
+    # https://docs.claude.com/en/docs/build-with-claude/citations#streaming-support
+    result$content[[i]]$citations <- c(
+      result$content[[i]]$citations,
+      list(chunk$delta$citation)
+    )
   } else if (chunk$type == "error") {
     if (chunk$error$type == "overloaded_error") {
       # https://docs.anthropic.com/en/api/messages-streaming#error-events
@@ -260,6 +334,16 @@ method(stream_merge_chunks, ProviderAnthropic) <- function(
   result
 }
 
+method(value_tokens, ProviderAnthropic) <- function(provider, json) {
+  tokens(
+    # Hack in pricing for cache writes
+    input = json$usage$input_tokens +
+      json$usage$cache_creation_input_tokens * 1.25,
+    output = json$usage$output_tokens,
+    cached_input = json$usage$cache_read_input_tokens
+  )
+}
+
 method(value_turn, ProviderAnthropic) <- function(
   provider,
   result,
@@ -270,13 +354,37 @@ method(value_turn, ProviderAnthropic) <- function(
       ContentText(content$text)
     } else if (content$type == "tool_use") {
       if (has_type) {
-        ContentJson(content$input$data)
+        ContentJson(data = content$input$data)
       } else {
         if (is_string(content$input)) {
           content$input <- jsonlite::parse_json(content$input)
         }
         ContentToolRequest(content$id, content$name, content$input)
       }
+    } else if (content$type == "server_tool_use") {
+      if (content$name == "web_search") {
+        # https://docs.claude.com/en/docs/agents-and-tools/tool-use/web-search-tool#response
+        ContentToolRequestSearch(
+          query = content$input$query,
+          json = content
+        )
+      } else if (content$name == "web_fetch") {
+        # https://docs.claude.com/en/docs/agents-and-tools/tool-use/web-fetch-tool#response
+        ContentToolRequestFetch(
+          url = content$input$url,
+          json = content
+        )
+      } else {
+        cli::cli_abort("Unknown server tool {.str {content$name}}.")
+      }
+    } else if (content$type == "web_search_tool_result") {
+      urls <- map_chr(content$content, \(x) x$url)
+      ContentToolResponseSearch(
+        url = urls,
+        json = content
+      )
+    } else if (content$type == "web_fetch_tool_result") {
+      ContentToolResponseFetch(url = content$url %||% "failed", json = content)
     } else if (content$type == "thinking") {
       ContentThinking(
         content$thinking,
@@ -290,29 +398,47 @@ method(value_turn, ProviderAnthropic) <- function(
     }
   })
 
-  tokens <- tokens_log(
-    provider,
-    input = result$usage$input_tokens,
-    output = result$usage$output_tokens
-  )
-
-  assistant_turn(contents, json = result, tokens = tokens)
+  tokens <- value_tokens(provider, result)
+  cost <- get_token_cost(provider, tokens)
+  AssistantTurn(contents, json = result, tokens = unlist(tokens), cost = cost)
 }
 
 # ellmer -> Claude --------------------------------------------------------------
 
-method(as_json, list(ProviderAnthropic, Turn)) <- function(provider, x) {
-  if (x@role == "system") {
+method(as_json, list(ProviderAnthropic, Turn)) <- function(
+  provider,
+  x,
+  ...,
+  is_last = FALSE
+) {
+  if (is_system_turn(x)) {
     # claude passes system prompt as separate arg
     NULL
-  } else if (x@role %in% c("user", "assistant")) {
-    list(role = x@role, content = as_json(provider, x@contents))
+  } else if (is_user_turn(x) || is_assistant_turn(x)) {
+    if (is_assistant_turn(x) && identical(x@contents, list())) {
+      # Drop empty assistant turns to avoid an API error
+      # (all messages must have non-empty content)
+      return(NULL)
+    }
+    x <- turn_contents_expand(x)
+    content <- as_json(provider, x@contents, ...)
+
+    # Add caching to the last content block in the last turn
+    # https://docs.claude.com/en/docs/build-with-claude/prompt-caching#how-automatic-prefix-checking-works
+    if (is_last) {
+      content[[length(content)]]$cache_control <- cache_control(provider)
+    }
+    list(role = x@role, content = content)
   } else {
-    cli::cli_abort("Unknown role {turn@role}", .internal = TRUE)
+    cli::cli_abort("Unknown role {x@role}", .internal = TRUE)
   }
 }
 
-method(as_json, list(ProviderAnthropic, ContentText)) <- function(provider, x) {
+method(as_json, list(ProviderAnthropic, ContentText)) <- function(
+  provider,
+  x,
+  ...
+) {
   if (is_whitespace(x@text)) {
     list(type = "text", text = "[empty string]")
   } else {
@@ -320,7 +446,11 @@ method(as_json, list(ProviderAnthropic, ContentText)) <- function(provider, x) {
   }
 }
 
-method(as_json, list(ProviderAnthropic, ContentPDF)) <- function(provider, x) {
+method(as_json, list(ProviderAnthropic, ContentPDF)) <- function(
+  provider,
+  x,
+  ...
+) {
   list(
     type = "document",
     source = list(
@@ -331,9 +461,40 @@ method(as_json, list(ProviderAnthropic, ContentPDF)) <- function(provider, x) {
   )
 }
 
-method(as_json, list(ProviderAnthropic, ContentImageRemote)) <- function(
+method(as_json, list(ProviderAnthropic, ContentUploaded)) <- function(
   provider,
   x
+) {
+  # https://docs.claude.com/en/docs/build-with-claude/files#using-a-file-in-messages
+  block_type <- switch(
+    x@mime_type,
+    "application/pdf" = "document",
+    "text/plain" = "document",
+    "image/jpeg" = "image",
+    "image/png" = "image",
+    "image/gif" = "image",
+    "image/webp" = "image",
+    "text/csv" = "container_upload",
+    "application/json" = "container_upload",
+    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet" = "container_upload",
+    "application/vnd.ms-excel" = "container_upload",
+    "text/xml" = "container_upload",
+    "application/xml" = "container_upload"
+  )
+
+  list(
+    type = block_type,
+    source = list(
+      type = "file",
+      file_id = x@uri
+    )
+  )
+}
+
+method(as_json, list(ProviderAnthropic, ContentImageRemote)) <- function(
+  provider,
+  x,
+  ...
 ) {
   list(
     type = "image",
@@ -346,7 +507,8 @@ method(as_json, list(ProviderAnthropic, ContentImageRemote)) <- function(
 
 method(as_json, list(ProviderAnthropic, ContentImageInline)) <- function(
   provider,
-  x
+  x,
+  ...
 ) {
   list(
     type = "image",
@@ -361,7 +523,8 @@ method(as_json, list(ProviderAnthropic, ContentImageInline)) <- function(
 # https://docs.anthropic.com/en/docs/build-with-claude/tool-use#handling-tool-use-and-tool-result-content-blocks
 method(as_json, list(ProviderAnthropic, ContentToolRequest)) <- function(
   provider,
-  x
+  x,
+  ...
 ) {
   list(
     type = "tool_use",
@@ -374,7 +537,8 @@ method(as_json, list(ProviderAnthropic, ContentToolRequest)) <- function(
 # https://docs.anthropic.com/en/docs/build-with-claude/tool-use#handling-tool-use-and-tool-result-content-blocks
 method(as_json, list(ProviderAnthropic, ContentToolResult)) <- function(
   provider,
-  x
+  x,
+  ...
 ) {
   list(
     type = "tool_result",
@@ -384,17 +548,22 @@ method(as_json, list(ProviderAnthropic, ContentToolResult)) <- function(
   )
 }
 
-method(as_json, list(ProviderAnthropic, ToolDef)) <- function(provider, x) {
+method(as_json, list(ProviderAnthropic, ToolDef)) <- function(
+  provider,
+  x,
+  ...
+) {
   list(
     name = x@name,
     description = x@description,
-    input_schema = compact(as_json(provider, x@arguments))
+    input_schema = compact(as_json(provider, x@arguments, ...))
   )
 }
 
 method(as_json, list(ProviderAnthropic, ContentThinking)) <- function(
   provider,
-  x
+  x,
+  ...
 ) {
   if (identical(x@thinking, "")) {
     return()
@@ -443,7 +612,7 @@ method(batch_submit, ProviderAnthropic) <- function(
 # https://docs.anthropic.com/en/api/retrieving-message-batches
 method(batch_poll, ProviderAnthropic) <- function(provider, batch) {
   req <- base_request(provider)
-  req <- req_url_path_append(req, "/messages/batches/", batch$id)
+  req <- req_url_path_append(req, "/messages/batches", batch$id)
   resp <- req_perform(req)
 
   resp_body_json(resp)
@@ -488,17 +657,11 @@ method(batch_result_turn, ProviderAnthropic) <- function(
   }
 }
 
-# Pricing ----------------------------------------------------------------------
-
-method(standardise_model, ProviderAnthropic) <- function(provider, model) {
-  gsub("-(latest|\\d{8})$", "", model)
-}
-
 # Models -----------------------------------------------------------------------
 
 #' @export
 #' @rdname chat_anthropic
-models_anthropic <- function(
+models_claude <- function(
   base_url = "https://api.anthropic.com/v1",
   api_key = anthropic_key()
 ) {
@@ -506,7 +669,8 @@ models_anthropic <- function(
     name = "Anthropic",
     model = "",
     base_url = base_url,
-    api_key = api_key
+    credentials = function() api_key,
+    cache = "none"
   )
 
   req <- base_request(provider)
@@ -524,14 +688,28 @@ models_anthropic <- function(
     name = display_name,
     created_at = created_at
   )
-  model_name <- standardise_model(provider, df$id)
-  df <- cbind(df, match_prices("Anthropic", model_name))
+  df <- cbind(df, match_prices("Anthropic", df$id))
   df[order(-xtfrm(df$created_at)), ]
 }
 
+#' @export
+#' @rdname chat_anthropic
+models_anthropic <- models_claude
+
 # Helpers ----------------------------------------------------------------
 
 # From httr2
 backoff_default <- function(i) {
   round(min(stats::runif(1, min = 1, max = 2^i), 60), 1)
 }
+
+cache_control <- function(provider) {
+  if (provider@cache == "none") {
+    NULL
+  } else {
+    list(
+      type = "ephemeral",
+      ttl = provider@cache
+    )
+  }
+}
