@@ -1,4 +1,4 @@
-#' @include provider-openai.R
+#' @include provider-openai-compatible.R
 NULL
 
 #' Chat with a model hosted on DeepSeek
@@ -6,6 +6,8 @@ NULL
 #' @description
 #' Sign up at <https://platform.deepseek.com>.
 #'
+#' Built on top of [chat_openai_compatible()].
+#'
 #' ## Known limitations
 #'
 #' * Structured data extraction is not supported.
@@ -14,7 +16,8 @@ NULL
 #' @export
 #' @family chatbots
 #' @inheritParams chat_openai
-#' @param api_key `r api_key_param("DEEPSEEK_API_KEY")`
+#' @param api_key `r lifecycle::badge("deprecated")` Use `credentials` instead.
+#' @param credentials `r api_key_param("DEEPSEEK_API_KEY")`
 #' @param base_url The base URL to the endpoint; the default uses DeepSeek.
 #' @param model `r param_model("deepseek-chat")`
 #' @inherit chat_openai return
@@ -26,10 +29,10 @@ NULL
 chat_deepseek <- function(
   system_prompt = NULL,
   base_url = "https://api.deepseek.com",
-  api_key = deepseek_key(),
+  api_key = NULL,
+  credentials = NULL,
   model = NULL,
   params = NULL,
-  seed = NULL,
   api_args = list(),
   echo = NULL,
   api_headers = character()
@@ -37,6 +40,13 @@ chat_deepseek <- function(
   model <- set_default(model, "deepseek-chat")
   echo <- check_echo(echo)
 
+  credentials <- as_credentials(
+    "chat_deepseek",
+    function() deepseek_key(),
+    credentials = credentials,
+    api_key = api_key
+  )
+
   params <- params %||% params()
 
   provider <- ProviderDeepSeek(
@@ -44,15 +54,17 @@ chat_deepseek <- function(
     base_url = base_url,
     model = model,
     params = params,
-    seed = seed,
     extra_args = api_args,
-    api_key = api_key,
+    credentials = credentials,
     extra_headers = api_headers
   )
   Chat$new(provider = provider, system_prompt = system_prompt, echo = echo)
 }
 
-ProviderDeepSeek <- new_class("ProviderDeepSeek", parent = ProviderOpenAI)
+ProviderDeepSeek <- new_class(
+  "ProviderDeepSeek",
+  parent = ProviderOpenAICompatible
+)
 
 method(chat_params, ProviderDeepSeek) <- function(provider, params) {
   # https://platform.deepseek.com/api-docs/api/create-chat-completion
@@ -71,20 +83,26 @@ method(chat_params, ProviderDeepSeek) <- function(provider, params) {
   )
 }
 
-method(as_json, list(ProviderDeepSeek, ContentText)) <- function(provider, x) {
+method(as_json, list(ProviderDeepSeek, ContentText)) <- function(
+  provider,
+  x,
+  ...
+) {
   x@text
 }
 
-method(as_json, list(ProviderDeepSeek, Turn)) <- function(provider, x) {
-  if (x@role == "user") {
+method(as_json, list(ProviderDeepSeek, Turn)) <- function(provider, x, ...) {
+  if (is_user_turn(x)) {
+    x <- turn_contents_expand(x)
+    data <- turn_split_tool_results(x)
+
     # Text and tool results go in separate messages
-    texts <- keep(x@contents, S7_inherits, ContentText)
+    texts <- keep(data$contents, S7_inherits, ContentText)
     texts_out <- lapply(texts, function(text) {
-      list(role = "user", content = as_json(provider, text))
+      list(role = "user", content = as_json(provider, text, ...))
     })
 
-    tools <- keep(x@contents, S7_inherits, ContentToolResult)
-    tools_out <- lapply(tools, function(tool) {
+    tools_out <- lapply(data$tool_results, function(tool) {
       list(
         role = "tool",
         content = tool_string(tool),
@@ -92,19 +110,19 @@ method(as_json, list(ProviderDeepSeek, Turn)) <- function(provider, x) {
       )
     })
 
-    c(texts_out, tools_out)
-  } else if (x@role == "assistant") {
+    c(tools_out, texts_out)
+  } else if (is_assistant_turn(x)) {
     # Tool requests come out of content and go into own argument
     text <- detect(x@contents, S7_inherits, ContentText)
     tools <- keep(x@contents, is_tool_request)
 
     list(compact(list(
       role = "assistant",
-      content = as_json(provider, text),
-      tool_calls = as_json(provider, tools)
+      content = as_json(provider, text, ...),
+      tool_calls = as_json(provider, tools, ...)
     )))
   } else {
-    as_json(super(provider, ProviderOpenAI), x)
+    as_json(super(provider, ProviderOpenAICompatible), x, ...)
   }
 }
 
