@@ -7,8 +7,7 @@
 #' {[ollamar](https://hauselin.github.io/ollama-r/)} (e.g.
 #' `ollamar::pull("llama3.1")`).
 #'
-#' This function is a lightweight wrapper around [chat_openai()] with
-#' the defaults tweaked for ollama.
+#' Built on top of [chat_openai_compatible()].
 #'
 #' ## Known limitations
 #'
@@ -22,12 +21,14 @@
 #'
 #' @inheritParams chat_openai
 #' @param model `r param_model(NULL, "ollama")`
-#' @param api_key Ollama doesn't require an API key for local usage and in most
-#'   cases you do not need to provide an `api_key`.
+#' @param api_key `r lifecycle::badge("deprecated")` Use `credentials` instead.
+#' @param credentials Ollama doesn't require credentials for local usage and in most
+#'   cases you do not need to provide `credentials`.
 #'
 #'   However, if you're accessing an Ollama instance hosted behind a reverse
 #'   proxy or secured endpoint that enforces bearer‐token authentication, you
-#'   can set `api_key` (or the `OLLAMA_API_KEY` environment variable).
+#'   can set the `OLLAMA_API_KEY` environment variable or provide a callback
+#'   function to `credentials`.
 #' @param params Common model parameters, usually created by [params()].
 #' @inherit chat_openai return
 #' @family chatbots
@@ -41,18 +42,22 @@ chat_ollama <- function(
   system_prompt = NULL,
   base_url = Sys.getenv("OLLAMA_BASE_URL", "http://localhost:11434"),
   model,
-  seed = NULL,
   params = NULL,
   api_args = list(),
   echo = NULL,
   api_key = NULL,
+  credentials = NULL,
   api_headers = character()
 ) {
-  if (!has_ollama(base_url)) {
+  # ollama doesn't require an API key for local usage, but one might be needed
+  # if ollama is served behind a proxy (see #501)
+  credentials <- ollama_credentials(credentials, api_key)
+
+  if (!has_ollama(base_url, credentials)) {
     cli::cli_abort("Can't find locally running ollama.")
   }
 
-  models <- models_ollama(base_url)$id
+  models <- models_ollama(base_url, credentials)$id
 
   if (missing(model)) {
     cli::cli_abort(c(
@@ -71,18 +76,13 @@ chat_ollama <- function(
 
   echo <- check_echo(echo)
 
-  params <- params %||% params()
-
   provider <- ProviderOllama(
     name = "Ollama",
     base_url = file.path(base_url, "v1"), ## the v1 portion of the path is added for openAI compatible API
     model = model,
-    seed = seed,
-    params = params,
+    params = params %||% params(),
     extra_args = api_args,
-    # ollama doesn't require an API key for local usage, but one might be needed
-    # if ollama is served behind a proxy (see #501)
-    api_key = api_key %||% Sys.getenv("OLLAMA_API_KEY", "ollama"),
+    credentials = credentials,
     extra_headers = api_headers
   )
 
@@ -91,14 +91,21 @@ chat_ollama <- function(
 
 ProviderOllama <- new_class(
   "ProviderOllama",
-  parent = ProviderOpenAI,
+  parent = ProviderOpenAICompatible,
   properties = list(
-    prop_redacted("api_key"),
-    model = prop_string(),
-    seed = prop_number_whole(allow_null = TRUE)
+    model = prop_string()
   )
 )
 
+ollama_credentials <- function(credentials = NULL, api_key = NULL) {
+  as_credentials(
+    "chat_ollama",
+    function() Sys.getenv("OLLAMA_API_KEY", ""),
+    credentials = credentials,
+    api_key = api_key
+  )
+}
+
 method(chat_params, ProviderOllama) <- function(provider, params) {
   # https://github.com/ollama/ollama/blob/main/docs/api.md#generate-a-chat-completion
   standardise_params(
@@ -135,8 +142,18 @@ skip_if_no_ollama <- function() {
 
 #' @export
 #' @rdname chat_ollama
-models_ollama <- function(base_url = "http://localhost:11434") {
+models_ollama <- function(
+  base_url = "http://localhost:11434",
+  credentials = NULL
+) {
+  credentials <- as_credentials(
+    "models_ollama",
+    function() Sys.getenv("OLLAMA_API_KEY", ""),
+    credentials = credentials
+  )
+
   req <- request(base_url)
+  req <- ellmer_req_credentials(req, credentials(), "Authorization")
   req <- req_url_path_append(req, "api/tags")
   resp <- req_perform(req)
   json <- resp_body_json(resp)
@@ -151,22 +168,26 @@ models_ollama <- function(base_url = "http://localhost:11434") {
     id = names,
     created_at = modified_at,
     size = size,
-    capabilities = ollama_model_capabilities(names)
+    capabilities = ollama_model_capabilities(base_url, names, credentials)
   )
   df[order(-xtfrm(df$created_at)), ]
 }
 
 the$ollama_cache <- new_environment()
 
-ollama_model_details <- function(model) {
+ollama_model_details <- function(
+  base_url,
+  model,
+  credentials = ollama_credentials()
+) {
   # https://github.com/ollama/ollama/blob/main/docs/api.md#show-model-information
-  url <- "http://localhost:11434/api/show"
-
   if (env_has(the$ollama_cache, model)) {
     return(the$ollama_cache[[model]])
   }
 
-  req <- request(url)
+  req <- request(base_url)
+  req <- ellmer_req_credentials(req, credentials(), "Authorization")
+  req <- req_url_path_append(req, "api/show")
   req <- req_body_json(req, list(model = model, verbose = FALSE))
 
   resp <- req_perform(req)
@@ -178,18 +199,31 @@ ollama_model_details <- function(model) {
   details
 }
 
-ollama_model_capabilities <- function(models) {
+ollama_model_capabilities <- function(
+  base_url,
+  models,
+  credentials = ollama_credentials()
+) {
   res <- map(models, function(m) {
-    tryCatch(ollama_model_details(m), error = function(e) NULL)
+    tryCatch(
+      ollama_model_details(base_url, m, credentials),
+      error = function(e) NULL
+    )
   })
   map_chr(res, \(x) paste(x$capabilities, collapse = ","))
 }
 
 
-has_ollama <- function(base_url = "http://localhost:11434") {
+has_ollama <- function(
+  base_url = "http://localhost:11434",
+  credentials = ollama_credentials()
+) {
+  check_credentials(credentials)
+
   tryCatch(
     {
       req <- request(base_url)
+      req <- ellmer_req_credentials(req, credentials(), "Authorization")
       req <- req_url_path_append(req, "api/tags")
       req_perform(req)
       TRUE
@@ -198,7 +232,11 @@ has_ollama <- function(base_url = "http://localhost:11434") {
   )
 }
 
-method(as_json, list(ProviderOllama, TypeObject)) <- function(provider, x) {
+method(as_json, list(ProviderOllama, TypeObject)) <- function(
+  provider,
+  x,
+  ...
+) {
   if (x@additional_properties) {
     cli::cli_abort("{.arg .additional_properties} not supported for Ollama.")
   }
@@ -209,7 +247,7 @@ method(as_json, list(ProviderOllama, TypeObject)) <- function(provider, x) {
   compact(list(
     type = "object",
     description = x@description %||% "",
-    properties = as_json(provider, x@properties),
+    properties = as_json(provider, x@properties, ...),
     required = as.list(names2(x@properties)[required]),
     additionalProperties = FALSE
   ))
