@@ -1,4 +1,4 @@
-#' @include provider-openai.R
+#' @include provider-openai-compatible.R
 #' @include content.R
 NULL
 
@@ -9,8 +9,11 @@ NULL
 #' provides an efficient and convenient LLMs model server. You can use
 #' `chat_vllm()` to connect to endpoints powered by vLLM.
 #'
+#' Uses OpenAI compatible API via `chat_openai_compatible()`.
+#'
 #' @inheritParams chat_openai
-#' @param api_key `r api_key_param("VLLM_API_KEY")`
+#' @param api_key `r lifecycle::badge("deprecated")` Use `credentials` instead.
+#' @param credentials `r api_key_param("VLLM_API_KEY")`
 #' @param model `r param_model(NULL, "vllm")`
 #' @param params Common model parameters, usually created by [params()].
 #' @inherit chat_openai return
@@ -24,17 +27,25 @@ chat_vllm <- function(
   base_url,
   system_prompt = NULL,
   model,
-  seed = NULL,
   params = NULL,
   api_args = list(),
-  api_key = vllm_key(),
+  api_key = NULL,
+  credentials = NULL,
   echo = NULL,
   api_headers = character()
 ) {
   check_string(base_url)
-  check_string(api_key)
+
+  credentials <- as_credentials(
+    "chat_vllm",
+    function() vllm_key(),
+    credentials = credentials,
+    api_key = api_key
+  )
+
+  check_string(credentials())
   if (missing(model)) {
-    models <- models_vllm(base_url, api_key)$id
+    models <- models_vllm(base_url, credentials = credentials)$id
     cli::cli_abort(c(
       "Must specify {.arg model}.",
       i = "Available models: {.str {models}}."
@@ -49,10 +60,9 @@ chat_vllm <- function(
     name = "VLLM",
     base_url = base_url,
     model = model,
-    seed = seed,
     params = params,
     extra_args = api_args,
-    api_key = api_key,
+    credentials = credentials,
     extra_headers = api_headers
   )
   Chat$new(provider = provider, system_prompt = system_prompt, echo = echo)
@@ -69,18 +79,18 @@ chat_vllm_test <- function(..., echo = "none") {
 
 ProviderVllm <- new_class(
   "ProviderVllm",
-  parent = ProviderOpenAI,
+  parent = ProviderOpenAICompatible,
   package = "ellmer",
 )
 
 # Just like OpenAI but no strict
-method(as_json, list(ProviderVllm, ToolDef)) <- function(provider, x) {
+method(as_json, list(ProviderVllm, ToolDef)) <- function(provider, x, ...) {
   list(
     type = "function",
     "function" = compact(list(
       name = x@name,
       description = x@description,
-      parameters = as_json(provider, x@arguments)
+      parameters = as_json(provider, x@arguments, ...)
     ))
   )
 }
@@ -91,9 +101,16 @@ vllm_key <- function() {
 
 #' @export
 #' @rdname chat_vllm
-models_vllm <- function(base_url, api_key = vllm_key()) {
+models_vllm <- function(base_url, api_key = NULL, credentials = NULL) {
+  credentials <- as_credentials(
+    "models_vllm",
+    function() vllm_key(),
+    credentials = credentials,
+    api_key = api_key
+  )
+
   req <- request(base_url)
-  req <- req_auth_bearer_token(req, api_key)
+  req <- req_auth_bearer_token(req, credentials())
   req <- req_url_path_append(req, "/v1/models")
   resp <- req_perform(req)
   json <- resp_body_json(resp)
