@@ -21,8 +21,8 @@ retry_test <- function(code, retries = 1) {
 
 test_params_stop <- function(chat_fun) {
   chat <- chat_fun(params = params(stop_sequences = "cool"))
-  out <- chat$chat("Repeat after the following phrase: Dogs are cool")
-  expect_equal(out, ellmer_output("Dogs are "))
+  out <- chat$chat("Repeat the following phrase: Dogs are cool")
+  expect_no_match(out, "cool")
 }
 
 # Tool calls -------------------------------------------------------------
@@ -49,6 +49,20 @@ test_tools_simple <- function(chat_fun) {
   expect_match(result, "February")
 }
 
+test_tool_image <- function(chat_fun) {
+  # has a subtle dependency on imagemagick
+  skip_on_cran()
+
+  chat <- chat_fun()
+  chat$register_tool(tool(
+    \() content_image_file(system.file("smol-animal.jpg", package = "ellmer")),
+    name = "draw_animal",
+    description = "Draw a cute animal"
+  ))
+  chat$chat("Draw a picture of a cute animal")
+  expect_match(chat$chat("What sort of animal is that?"), "kitten|cat")
+}
+
 # Data extraction --------------------------------------------------------
 
 test_data_extraction <- function(chat_fun) {
@@ -81,6 +95,35 @@ test_data_extraction <- function(chat_fun) {
   )
 }
 
+# Built-in tools ---------------------------------------------------------
+
+test_tool_web_fetch <- function(chat_fun, tool) {
+  chat <- chat_fun()
+  chat$register_tool(tool)
+
+  url <- "https://rvest.tidyverse.org/articles/starwars.html"
+  expect_match(
+    chat$chat(paste0("What's the first movie listed on ", url, "?")),
+    "The Phantom Menace"
+  )
+  expect_match(chat$chat("Who directed it?"), "George Lucas")
+}
+
+test_tool_web_search <- function(chat_fun, tool, hint = NULL) {
+  chat <- chat_fun()
+  chat$register_tool(tool)
+
+  result <- chat$chat(c(
+    "When was ggplot2 1.0.0 released to CRAN?",
+    "Answer in YYYY-MM-DD format.",
+    hint
+  ))
+  # for openAI
+  result <- gsub("\u2011", "-", result, fixed = TRUE)
+  expect_match(result, "2014-05-21")
+  expect_match(chat$chat("What month was that?"), "May")
+}
+
 # Images -----------------------------------------------------------------
 
 test_images_inline <- function(chat_fun, test_shape = TRUE) {
