@@ -4,7 +4,10 @@ test_that("can make simple request", {
   chat <- chat_openai_test()
   resp <- chat$chat("What is 1 + 1?", echo = FALSE)
   expect_match(resp, "2")
-  expect_equal(chat$last_turn()@tokens[1:2] > 0, c(TRUE, TRUE))
+  expect_equal(unname(chat$last_turn()@tokens[1:2] > 0), c(TRUE, TRUE))
+
+  resp <- chat$chat("Double that", echo = FALSE)
+  expect_match(resp, "4")
 })
 
 test_that("can make simple streaming request", {
@@ -17,34 +20,50 @@ test_that("can list models", {
   test_models(models_openai)
 })
 
-
 # Common provider interface -----------------------------------------------
 
 test_that("defaults are reported", {
   expect_snapshot(. <- chat_openai())
 })
 
-test_that("supports standard parameters", {
-  chat_fun <- chat_openai_test
+# No longer supports stop parameter
+# test_that("supports standard parameters", {
+#   chat_fun <- chat_openai_test
 
-  test_params_stop(chat_fun)
-})
+#   test_params_stop(chat_fun)
+# })
 
 test_that("supports tool calling", {
-  vcr::local_cassette("openai-tool")
+  vcr::local_cassette("openai-v2-tool")
   chat_fun <- chat_openai_test
 
   test_tools_simple(chat_fun)
 })
 
+test_that("tools can return images", {
+  vcr::local_cassette("openai-v2-tool-image")
+  chat_fun <- chat_openai_test
+  test_tool_image(chat_fun)
+})
+
 test_that("can extract data", {
   chat_fun <- chat_openai_test
 
   test_data_extraction(chat_fun)
 })
 
+test_that("can search web pages", {
+  vcr::local_cassette("openai-v2-web-search")
+  chat_fun <- \(...) chat_openai_test(model = "gpt-4.1", ...)
+  test_tool_web_search(
+    chat_fun,
+    openai_tool_web_search(),
+    hint = "The CRAN archive page has this info."
+  )
+})
+
 test_that("can use images", {
-  vcr::local_cassette("openai-image")
+  vcr::local_cassette("openai-v2-image")
   # Needs mini to get shape correct
   chat_fun <- \(...) chat_openai_test(model = "gpt-4.1-mini", ...)
 
@@ -53,7 +72,7 @@ test_that("can use images", {
 })
 
 test_that("can use pdfs", {
-  vcr::local_cassette("openai-pdf")
+  vcr::local_cassette("openai-v2-pdf")
   chat_fun <- chat_openai_test
 
   test_pdf_local(chat_fun)
@@ -70,13 +89,8 @@ test_that("can match prices for some common models", {
 
 test_that("can retrieve log_probs (#115)", {
   chat <- chat_openai_test(params = params(log_probs = TRUE))
-  pieces <- coro::collect(chat$stream("Hi"))
-
-  logprobs <- chat$last_turn()@json$choices[[1]]$logprobs$content
-  expect_equal(
-    length(logprobs),
-    length(pieces) - 2 # leading "" + trailing \n
-  )
+  chat$chat("Hi")
+  expect_length(chat$last_turn()@json$output[[1]]$content[[1]]$logprobs, 2)
 })
 
 test_that("structured data work with and without wrapper", {
@@ -94,33 +108,37 @@ test_that("structured data work with and without wrapper", {
   expect_equal(out, list(number = 11))
 })
 
-# Custom -----------------------------------------------------------------
+test_that("service tier affects pricing", {
+  vcr::local_cassette("openai-v2-service-tier")
+  chat <- chat_openai_test(service_tier = "priority")
+  chat$chat("Tell me a joke")
 
-test_that("as_json specialised for OpenAI", {
-  stub <- ProviderOpenAI(name = "", base_url = "", api_key = "", model = "")
+  last_turn <- chat$last_turn()
+  tokens <- as.list(last_turn@tokens)
+  priority_cost <- get_token_cost(chat$get_provider(), tokens, "priority")
+  expect_equal(last_turn@cost, priority_cost)
 
-  expect_snapshot(
-    as_json(stub, type_object(.additional_properties = TRUE)),
-    error = TRUE
+  # Confirm we have pricing for the priority tier
+  default_cost <- get_token_cost(chat$get_provider(), tokens)
+  expect_gt(last_turn@cost, default_cost)
+})
+
+
+test_that("batch retrieve succeeds even if JSON is mangled", {
+  local_mocked_bindings(
+    openai_download_file = function(provider, id, path) {
+      writeLines('{"custom_id": "123", ', path)
+    }
   )
+  provider <- chat_openai_test()$get_provider()
+  out <- batch_retrieve(provider, list(output_file_id = "123"))
+  expect_equal(out, list(list(status_code = 500)))
+  expect_equal(batch_result_turn(provider, out[[1]]), NULL)
+})
 
-  obj <- type_object(x = type_number(required = FALSE))
+test_that("can extract dummy response from malformed JSON", {
   expect_equal(
-    as_json(stub, obj),
-    list(
-      type = "object",
-      description = "",
-      properties = list(x = list(type = c("number", "null"), description = "")),
-      required = list("x"),
-      additionalProperties = FALSE
-    )
+    openai_json_fallback('{"custom_id": "123", '),
+    list(custom_id = "123", response = list(status_code = 500))
   )
 })
-
-test_that("seed is deprecated, but still honored", {
-  expect_snapshot(chat <- chat_openai_test(seed = 1))
-  expect_equal(chat$get_provider()@params$seed, 1)
-
-  # NULL is also ignored since that's what subclasses use
-  expect_no_warning(chat_openai_test(seed = NULL))
-})
