@@ -1,7 +1,25 @@
+test_that("Turn() with `role`", {
+  suppressWarnings({
+    # Promotes system, user, assistant to subclasses of Turn
+    expect_s7_class(Turn(role = "system"), SystemTurn)
+    expect_s7_class(Turn(role = "user"), UserTurn)
+    expect_s7_class(Turn(role = "assistant"), AssistantTurn)
+  })
+
+  # No role: unknown
+  expect_equal(Turn()@role, "unknown")
+
+  # Invalid roles are errors
+  expect_snapshot(error = TRUE, {
+    Turn(role = "invalid")
+    Turn(role = 42)
+  })
+})
+
 test_that("system prompt is applied correctly", {
   sys_prompt <- "foo"
-  sys_msg <- Turn("system", sys_prompt)
-  user_msg <- Turn("user", "bar")
+  sys_msg <- SystemTurn(sys_prompt)
+  user_msg <- UserTurn("bar")
 
   expect_equal(normalize_turns(), list())
   expect_equal(normalize_turns(list(user_msg)), list(user_msg))
@@ -20,8 +38,8 @@ test_that("system prompt is applied correctly", {
 
 test_that("normalize_turns throws useful errors", {
   sys_prompt <- "foo"
-  sys_msg <- Turn("system", "foo")
-  user_msg <- Turn("user", "bar")
+  sys_msg <- SystemTurn("foo")
+  user_msg <- UserTurn("bar")
 
   expect_snapshot(error = TRUE, {
     normalize_turns(1)
@@ -42,7 +60,7 @@ test_that("as_user_turn gives useful errors", {
 
 test_that("can opt-out of empty check", {
   out <- as_user_turn(list(), check_empty = FALSE)
-  expect_equal(out, Turn("user"))
+  expect_equal(out, UserTurn())
 })
 
 test_that("as_user_turns gives useful errors", {
@@ -54,8 +72,7 @@ test_that("as_user_turns gives useful errors", {
 })
 
 test_that("can extract text easily", {
-  turn <- Turn(
-    "assistant",
+  turn <- AssistantTurn(
     list(
       ContentText("ABC"),
       ContentImage(),
@@ -66,7 +83,7 @@ test_that("can extract text easily", {
 })
 
 test_that("turns have a reasonable print method", {
-  expect_snapshot(Turn("user", "hello"))
+  expect_snapshot(UserTurn("hello"))
 })
 
 test_that("as_user_turns can create lists of turns from lists of Content objects", {
@@ -81,3 +98,39 @@ test_that("as_user_turns can create lists of turns from lists of Content objects
   expect_s3_class(content_turns[[1]], "ellmer::Turn")
   expect_s3_class(content_turns[[2]], "ellmer::Turn")
 })
+
+# turn_contents_preview()
+
+test_that("ContentText shows first (truncated) text", {
+  expect_equal(
+    turn_contents_preview(UserTurn("This is short")),
+    "Text[This is short]"
+  )
+  expect_equal(
+    turn_contents_preview(UserTurn(
+      "This is a very long message that should be truncated"
+    )),
+    "Text[This is a very long message that shou...]"
+  )
+
+  expect_equal(
+    turn_contents_preview(UserTurn(
+      list(ContentText("This is short"), ContentText("Second"))
+    )),
+    "Text[This is short], Text"
+  )
+})
+
+test_that("non-text types just show type", {
+  expect_equal(
+    turn_contents_preview(UserTurn(list(ContentImageInline("")))),
+    "ImageInline"
+  )
+
+  expect_equal(
+    turn_contents_preview(UserTurn(
+      list(ContentImageInline(""), ContentImageRemote(""))
+    )),
+    "ImageInline, ImageRemote"
+  )
+})
