@@ -8,13 +8,34 @@ test_that("prop_whole_number validates inputs", {
   }
   expect_snapshot(error = TRUE, {
     check_prop()("x")
-    check_prop()(c(1:2))
+    check_prop()(c(1, 2))
     check_prop()(1.5)
     check_prop(min = 1)(0)
     check_prop(max = -1)(0)
+    check_prop()(NA_real_)
   })
 })
 
+test_that("prop_whole_decimal validates inputs", {
+  check_prop <- function(...) {
+    new_class(
+      "class",
+      properties = list(prop = prop_number_decimal(...)),
+      package = NULL
+    )
+  }
+  expect_snapshot(error = TRUE, {
+    check_prop()("x")
+    check_prop()(c(1, 2))
+    check_prop()(1.5)
+    check_prop(min = 1)(0)
+    check_prop(max = -1)(0)
+    check_prop(max = -1)(Inf)
+    check_prop()(NA_real_)
+  })
+})
+
+
 test_that("redacted values aren't saved to disk", {
   Test <- new_class("Test", properties = list(prop_redacted("redacted")))
 
